@@ -1,15 +1,15 @@
-/********************************************************************************/
-/* Copyright (c) 2004                                                           */
-/* Daniel Sleator, David Temperley, and John Lafferty                           */
-/* All rights reserved                                                          */
-/*                                                                              */
-/* Use of the link grammar parsing system is subject to the terms of the        */
-/* license set forth in the LICENSE file included with this software,           */ 
-/* and also available at http://www.link.cs.cmu.edu/link/license.html           */
-/* This license allows free redistribution and use in source and binary         */
-/* forms, with or without modification, subject to certain conditions.          */
-/*                                                                              */
-/********************************************************************************/
+/*************************************************************************/
+/* Copyright (c) 2004                                                    */
+/* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* All rights reserved                                                   */
+/*                                                                       */
+/* Use of the link grammar parsing system is subject to the terms of the */
+/* license set forth in the LICENSE file included with this software,    */ 
+/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* This license allows free redistribution and use in source and binary  */
+/* forms, with or without modification, subject to certain conditions.   */
+/*                                                                       */
+/*************************************************************************/
 
 /*****************************************************************************
 *
@@ -191,7 +191,7 @@ struct Sentence_s {
 typedef struct DTreeLeaf_s DTreeLeaf;
 typedef struct Domain_s Domain;
 struct Domain_s {
-  char *         string;
+  const char *   string;
   int            size;
   List_o_links * lol;
   int            start_link;  /* the link that started this domain */
@@ -251,14 +251,14 @@ struct Sublinkage_s {
     int       num_links;          /* Number of links in array */
     Link *    link;               /* Array of links */
     PP_info * pp_info;            /* PP info for each link */
-    char *    violation;          /* Name of violation, if any */
+    const char * violation;       /* Name of violation, if any */
     PP_data   pp_data;
 };
 
 
 struct Linkage_s {
     int             num_words;  /* number of (tokenized) words */
-    char *        * word;       /* array of word spellings */
+    const char *  * word;       /* array of word spellings */
     Linkage_info    info;       /* index and cost information */
     int             num_sublinkages; /* One for thin linkages, bigger for fat */
     int             current;    /* Allows user to select particular sublinkage */
