@@ -253,10 +253,14 @@ int parse_options_get_display_postscript(Parse_Options opts) {
 	return opts->display_postscript;
 }
 
-void parse_options_set_display_constituents(Parse_Options opts, int dummy) {
-	if ((dummy < 0) || (dummy > 2)) {
+void parse_options_set_display_constituents(Parse_Options opts, int dummy)
+{
+	if ((dummy < 0) || (dummy > 3)) {
 	   fprintf(stderr, "Possible values for constituents: \n");
-	   fprintf(stderr, "   0 (no display) 1 (treebank style) or 2 (flat tree)\n");
+	   fprintf(stderr, "   0 (no display)\n");
+	   fprintf(stderr, "   1 (treebank style, multi-line indented)\n");
+	   fprintf(stderr, "   2 (flat tree, square brackets)\n");
+	   fprintf(stderr, "   3 (flat treebank style)\n");
 	   opts->display_constituents = 0;
 	}
 	else opts->display_constituents = dummy;
@@ -574,6 +578,7 @@ static void free_post_processing(Sentence sent)
 
 void sentence_delete(Sentence sent) 
 {
+	if (!sent) return;
 	/* free_andlists(sent); */
 	free_sentence_disjuncts(sent);
 	free_sentence_expressions(sent);
@@ -594,38 +599,57 @@ int sentence_length(Sentence sent)
 }
 
 char * sentence_get_word(Sentence sent, int index) {
+	if (!sent) return NULL;
 	return sent->word[index].string;
 }
 
 int sentence_null_count(Sentence sent) {
+	if (!sent) return 0;
 	return sent->null_count;
 }
 
 int sentence_num_linkages_found(Sentence sent) {
+	if (!sent) return 0;
 	return sent->num_linkages_found;
 }
 
 int sentence_num_valid_linkages(Sentence sent) {
+	if (!sent) return 0;
 	return sent->num_valid_linkages;
 }
 
 int sentence_num_linkages_post_processed(Sentence sent) {
+	if (!sent) return 0;
 	return sent->num_linkages_post_processed;
 }
 
 int sentence_num_violations(Sentence sent, int i) {
+	if (!sent) return 0;
 	return sent->link_info[i].N_violations;
 }
 
+int sentence_and_cost(Sentence sent, int i) {
+	if (!sent) return 0;
+	return sent->link_info[i].and_cost;
+}
+
 int sentence_disjunct_cost(Sentence sent, int i) {
+	if (!sent) return 0;
 	return sent->link_info[i].disjunct_cost;
 }
 
+int sentence_link_cost(Sentence sent, int i) {
+	if (!sent) return 0;
+	return sent->link_info[i].link_cost;
+}
+
 char * sentence_get_nth_word(Sentence sent, int i) {
+	if (!sent) return NULL;
 	return sent->word[i].string;
 }
 
 int sentence_nth_word_has_disjunction(Sentence sent, int i) {
+	if (!sent) return 0;
 	return (sent->parse_info->chosen_disjuncts[i] != NULL);
 }
 
