@@ -37,6 +37,8 @@ static const char * msg_of_lperror(int lperr)
 		return "Error separating sentence ";
 	case NOTINDICT:
 		return "Sentence not in dictionary ";
+	case CHARSET:
+		return "Unable to process input string in current locale ";
 	case BUILDEXPR:
 		return "Could not build sentence expressions ";
 	case INTERNALERROR:
@@ -57,7 +59,7 @@ void lperror(int lperr, const char *fmt, ...)
 	va_end(args);
 
 	strncpy(lperrmsg, msg_of_lperror(lperr), MSGSZ);
-	strncat(lperrmsg, temp, MSGSZ);
+	strncat(lperrmsg, temp, MSGSZ-strlen(lperrmsg)-1);
 	lperrmsg[MSGSZ-1]=0; /* Null terminate at all costs */
 	lperrno = lperr;
 }
