@@ -5,9 +5,6 @@
  * It could be improved.
  */
 
-/* #define BUILD_JNI_CLIENT 1 */
-#ifdef BUILD_JNI_CLIENT
-
 #include <jni.h>
 #include <stdio.h>
 #include <utilities.h>
@@ -339,6 +336,17 @@ Java_org_linkgrammar_LinkGrammar_getLinkageNumViolations(JNIEnv *env, jclass cls
 	return sentence_num_violations(sent, cur_linkage);
 }
 
+/*
+ * Class:		 LinkGrammar
+ * Method:		linkageAndCost
+ * Signature: ()I
+ */
+JNIEXPORT jint JNICALL
+Java_org_linkgrammar_LinkGrammar_getLinkageAndCost(JNIEnv *env, jclass cls)
+{
+	return sentence_and_cost(sent, cur_linkage);
+}
+
 /*
  * Class:		 LinkGrammar
  * Method:		linkageDisjunctCost
@@ -350,6 +358,17 @@ Java_org_linkgrammar_LinkGrammar_getLinkageDisjunctCost(JNIEnv *env, jclass cls)
 	return sentence_disjunct_cost(sent, cur_linkage);
 }
 
+/*
+ * Class:		 LinkGrammar
+ * Method:		linkageLinkCost
+ * Signature: ()I
+ */
+JNIEXPORT jint JNICALL
+Java_org_linkgrammar_LinkGrammar_getLinkageLinkCost(JNIEnv *env, jclass cls)
+{
+	return sentence_link_cost(sent, cur_linkage);
+}
+
 /*
  * Class:		 LinkGrammar
  * Method:		getNumLinks
@@ -485,5 +504,3 @@ Java_org_linkgrammar_LinkGrammar_isEntity(JNIEnv *env, jclass cls, jstring str)
 		return TRUE;
 	return FALSE;
 }
-
-#endif /* BUILD_JNI_CLIENT */
