@@ -106,6 +106,14 @@ JNIEXPORT void JNICALL Java_org_linkgrammar_LinkGrammar_makeLinkage
 JNIEXPORT jint JNICALL Java_org_linkgrammar_LinkGrammar_getLinkageNumViolations
 	(JNIEnv *, jclass);
 
+/*
+ * Class:     LinkGrammar
+ * Method:    linkageAndCost
+ * Signature: ()I
+ */
+JNIEXPORT jint JNICALL Java_org_linkgrammar_LinkGrammar_getLinkageAndCost
+	(JNIEnv *, jclass);
+
 /*
  * Class:     LinkGrammar
  * Method:    linkageDisjunctCost
@@ -114,6 +122,14 @@ JNIEXPORT jint JNICALL Java_org_linkgrammar_LinkGrammar_getLinkageNumViolations
 JNIEXPORT jint JNICALL Java_org_linkgrammar_LinkGrammar_getLinkageDisjunctCost
 	(JNIEnv *, jclass);
 
+/*
+ * Class:     LinkGrammar
+ * Method:    linkageLinkCost
+ * Signature: ()I
+ */
+JNIEXPORT jint JNICALL Java_org_linkgrammar_LinkGrammar_getLinkageLinkCost
+	(JNIEnv *, jclass);
+
 /*
  * Class:     LinkGrammar
  * Method:    numLinks
