@@ -142,7 +142,7 @@ static wint_t get_character(Dictionary dict, int quote_mode)
 
 
 /*
-   This set of 10 characters are the ones defining the syntax of the dictionary.
+ * This set of 10 characters are the ones defining the syntax of the dictionary.
 */
 #define SPECIAL "(){};[]&|:"
 
@@ -175,7 +175,7 @@ static int link_advance(Dictionary dict)
 
 	i = 0;
 	for (;;) {
-		if (i > MAX_TOKEN_LENGTH-3) {  /* 3 for multui-byte tokens */
+		if (i > MAX_TOKEN_LENGTH-3) {  /* 3 for multi-byte tokens */
 			dict_error(dict, "Token too long");
 			return 0;
 		}
@@ -189,6 +189,7 @@ static int link_advance(Dictionary dict)
 				dict_error(dict, "White space inside of token");
 				return 0;
 			}
+			/* store UTF8 internally, always. */
 			i += wctomb(&dict->token[i], c);
 		} else {
 			if (strchr(SPECIAL, c) != NULL) {
@@ -218,8 +219,8 @@ static int link_advance(Dictionary dict)
 			if (c == '\"') {
 				quote_mode = TRUE;
 			} else {
-				dict->token[i] = c;
-				i++;
+				/* store UTF8 internally, always. */
+				i += wctomb(&dict->token[i], c);
 			}
 		}
 		c = get_character(dict, quote_mode);
@@ -637,27 +638,35 @@ int dict_compare(char *s, char *t) {
 */
 
 /* terse version */
-static int dict_compare(const char *s, const char *t) {
+static int dict_compare(const char *s, const char *t)
+{
 	while (*s != '\0' && *s == *t) {s++; t++;}
 	return (((*s == '.')?(1):((*s)<<1))  -  ((*t == '.')?(1):((*t)<<1)));
 }
 
-Dict_node * insert_dict(Dictionary dict, Dict_node * n, Dict_node * new) {
-/* Insert the new node into the dictionary below node n                  */
-/* give error message if the new element's string is already there       */
-/* assumes that the "n" field of new is already set, and the left        */
-/* and right fields of it are NULL                                       */
+/**
+ * Insert the new node into the dictionary below node n.
+ * Give error message if the new element's string is already there.
+ * Assumes that the "n" field of new is already set, and the left
+ * and right fields of it are NULL.
+ *
+ * XXX The resulting tree is rather highly unbalanced. It would improve
+ * performance of rdictionary_lookup() by a factor of two if a tree
+ * balancing step was run after the dictionary creation. XXX
+ */
+Dict_node * insert_dict(Dictionary dict, Dict_node * n, Dict_node * newnode)
+{
 	int comp;
 	char t[128];
 
-	if (n == NULL) return new;
-	comp = dict_compare(new->string, n->string);
+	if (n == NULL) return newnode;
+	comp = dict_compare(newnode->string, n->string);
 	if (comp < 0) {
-		n->left = insert_dict(dict, n->left, new);
+		n->left = insert_dict(dict, n->left, newnode);
 	} else if (comp > 0) {
-		n->right = insert_dict(dict, n->right, new);
+		n->right = insert_dict(dict, n->right, newnode);
 	} else {
-		sprintf(t, "The word \"%s\" has been multiply defined\n", new->string);
+		sprintf(t, "The word \"%s\" has been multiply defined\n", newnode->string);
 		dict_error(dict, t);
 		return NULL;
 	}
@@ -721,9 +730,12 @@ static void insert_list(Dictionary dict, Dict_node * p, int l)
 	insert_list(dict, dn_second_half, l-k-1);
 }
 
-static int read_entry(Dictionary dict) {
-/* Starting with the current token parse one dictionary entry.         */
-/* Add these words to the dictionary                                   */
+/**
+ * Starting with the current token parse one dictionary entry.
+ * Add these words to the dictionary.
+ */
+static int read_entry(Dictionary dict)
+{
     Exp *n;
     int i;
 
@@ -812,7 +824,8 @@ void print_dictionary_data(Dictionary dict) {
 }
 
 
-int read_dictionary(Dictionary dict) {
+int read_dictionary(Dictionary dict)
+{
 	lperrno = 0;
 	if (!link_advance(dict)) {
 		fclose(dict->fp);
@@ -913,7 +926,12 @@ void free_lookup_list(Dict_node *llist)
 	}
 }
 
-static Dict_node * rdictionary_lookup(Dict_node *llist, Dict_node * dn, const char * s)
+/**
+ * rdictionary_lookup() --recursive dictionary lookup
+ * Walks binary tree.
+ */
+static Dict_node * rdictionary_lookup(Dict_node *llist, 
+                                      Dict_node * dn, const char * s)
 {
 	/* see comment in dictionary_lookup below */
 	int m;
@@ -1037,14 +1055,15 @@ static int find_one_non_idiom_node(Dict_node * p, Dict_node * dn, const char * s
 }
 
 static void set_parent_of_node(Dictionary dict,
-						Dict_node *p, Dict_node * del, Dict_node * new) {
+						Dict_node *p, Dict_node * del, Dict_node * newnode)
+{
 	if (p == NULL) {
-		dict->root = new;
+		dict->root = newnode;
 	} else {
 		if (p->left == del) {
-			p->left = new;
+			p->left = newnode;
 		} else if (p->right == del) {
-			p->right = new;
+			p->right = newnode;
 		} else {
 			assert(FALSE, "Dictionary broken?");
 		}
