@@ -11,6 +11,10 @@
 /*                                                                       */
 /*************************************************************************/
 
+#ifndef _WIN32
+#include <langinfo.h>
+#endif
+#include <limits.h>
 #include <link-grammar/api.h>
 
 #define MAX_STRIP 10
@@ -216,8 +220,8 @@ static int issue_sentence_word(Sentence sent, const char * s)
 static int downcase_is_in_dict(Dictionary dict, char * word)
 {
 	int i, rc;
-	char low[MB_CUR_MAX];
-	char save[MB_CUR_MAX];
+	char low[MB_LEN_MAX];
+	char save[MB_LEN_MAX];
 	wchar_t c;
 	int nbl, nbh;
 
@@ -305,9 +309,10 @@ static int separate_word(Sentence sent, char *w, char *wend, int is_first_word,
 				prefix[l] = dn->string;
 				l++;
 			}
-			dn2=dn->right;
+			dn2 = dn->right;
+			dn->right = NULL;
 			xfree(dn, sizeof(Dict_node));
-			dn=dn2;
+			dn = dn2;
 		}
 	}
 
@@ -493,6 +498,9 @@ int separate_sentence(char * s, Sentence sent)
 	if (dict->left_wall_defined)
 		if (!issue_sentence_word(sent, LEFT_WALL_WORD)) return FALSE;
 
+	/* Reset the multibyte shift state to the initial state */
+	mbtowc(NULL, NULL, 4);
+
 	is_first = TRUE;
 	for(;;) 
 	{
@@ -500,22 +508,27 @@ int separate_sentence(char * s, Sentence sent)
 		int nb = mbtowc(&c, s, 4);
 		quote_found = FALSE;
 
+		if (0 > nb) goto failure;
+
 		/* skip all whitespace */
 		while (iswspace(c) || (c == '\"'))
 		{
 			s += nb;
 			if(*s == '\"') quote_found=TRUE;
 			nb = mbtowc(&c, s, 4);
+			if (0 > nb) goto failure;
 		}
 
 		if (*s == '\0') break;
 
 		t = s;
-		nb = mbtowc(&c, t, 4);
-		while (!iswspace(c) && (c != '\"') && (c != 0))
+		nb = mbtowc(&c, t, 6);
+		if (0 > nb) goto failure;
+		while (!iswspace(c) && (c != '\"') && (c != 0) && (nb != 0))
 		{
 			t += nb;
 			nb = mbtowc(&c, t, 4);
+			if (0 > nb) goto failure;
 		}
 
 		if (!separate_word(sent, s, t, is_first, quote_found)) return FALSE;
@@ -528,6 +541,12 @@ int separate_sentence(char * s, Sentence sent)
 		if (!issue_sentence_word(sent, RIGHT_WALL_WORD)) return FALSE;
 
 	return (sent->length > dict->left_wall_defined + dict->right_wall_defined);
+
+failure:
+#ifndef _WIN32
+	lperror(CHARSET, "%s\n", nl_langinfo(CODESET));
+#endif
+	return FALSE;
 }
 
 static int special_string(Sentence sent, int i, const char * s) {
@@ -660,32 +679,60 @@ int build_sentence_expressions(Sentence sent)
 	for (i=0; i<sent->length; i++)
 	{
 		s = sent->word[i].string;
-		if (boolean_dictionary_lookup(sent->dict, s)) {
+		if (boolean_dictionary_lookup(sent->dict, s))
+		{
 			sent->word[i].x = build_word_expressions(sent, s);
-		} else if (is_utf8_upper(s) && is_s_word(s) && dict->pl_capitalized_word_defined) {
+		}
+		else if (is_utf8_upper(s) && is_s_word(s) && dict->pl_capitalized_word_defined) 
+		{
 			if (!special_string(sent, i, PL_PROPER_WORD)) return FALSE;
-		} else if (is_utf8_upper(s) && dict->capitalized_word_defined) {
+		}
+		else if (is_utf8_upper(s) && dict->capitalized_word_defined)
+		{
 			if (!special_string(sent, i, PROPER_WORD)) return FALSE;
-		} else if (is_number(s) && dict->number_word_defined) {
+		}
+		else if (is_number(s) && dict->number_word_defined)
+		{
 			/* we know it's a plural number, or 1 */
 			/* if the string is 1, we'll only be here if 1's not in the dictionary */
 			if (!special_string(sent, i, NUMBER_WORD)) return FALSE;
-		} else if (ishyphenated(s) && dict->hyphenated_word_defined) {
+		}
+		else if (ishyphenated(s) && dict->hyphenated_word_defined)
+		{
 			/* singular hyphenated */
 			if (!special_string(sent, i, HYPHENATED_WORD)) return FALSE;
-		} else if (is_ing_word(s) && dict->ing_word_defined) {
+		} 
+		/* XXX
+		 * The following does some morphology-guessing for words that
+		 * that are not in the dictionary. This should be replaced by
+		 * a generic morphology-guesser for langauges that aren't english.
+		 * XXX
+		 */
+		else if (is_ing_word(s) && dict->ing_word_defined) 
+		{
 			if (!guessed_string(sent, i, s, ING_WORD)) return FALSE;
-		} else if (is_s_word(s) && dict->s_word_defined) {
+		}
+		else if (is_s_word(s) && dict->s_word_defined)
+		{
 			if (!guessed_string(sent, i, s, S_WORD)) return FALSE;
-		} else if (is_ed_word(s) && dict->ed_word_defined) {
+		}
+		else if (is_ed_word(s) && dict->ed_word_defined)
+		{
 			if (!guessed_string(sent, i, s, ED_WORD)) return FALSE;
-		} else if (is_ly_word(s) && dict->ly_word_defined) {
+		}
+		else if (is_ly_word(s) && dict->ly_word_defined)
+		{
 			if (!guessed_string(sent, i, s, LY_WORD)) return FALSE;
-		} else if (dict->unknown_word_defined && dict->use_unknown_word) {
+		}
+		else if (dict->unknown_word_defined && dict->use_unknown_word)
+		{
 			handle_unknown_word(sent, i, s);
-		} else {
-			/* the reason I can assert this is that the word
-			   should have been looked up already if we get here. */
+		}
+		else 
+		{
+			/* The reason I can assert this is that the word
+			 * should have been looked up already if we get here.
+			 */
 			assert(FALSE, "I should have found that word.");
 		}
 	}
@@ -695,7 +742,8 @@ int build_sentence_expressions(Sentence sent)
 	 * capitalized has to be looked up as an uncapitalized word
 	 * (as well as a capitalized word).
 	 */
-	for (i=0; i<sent->length; i++) {
+	for (i=0; i<sent->length; i++)
+	{
 		if (! (i==first_word || (i>0 && strcmp(":", sent->word[i-1].string)==0) || post_quote[i]==1) ) continue;
 		s = sent->word[i].string;
 
