@@ -12,6 +12,7 @@
 /*************************************************************************/
 
 #include <link-grammar/api.h>
+#include <limits.h>
 #include <locale.h>
 #include <stdlib.h>
 #include <string.h>
@@ -34,9 +35,11 @@
 #define DEFAULTPATH DICTIONARY_DIR
 
 /* This file contains certain general utilities. */
-
 int   verbosity;
 
+/* ============================================================= */
+/* String utilities */
+
 char *safe_strdup(const char *u)
 {
 	if(u)
@@ -65,6 +68,67 @@ void safe_strcat(char *u, const char *v, size_t usize)
 	u[usize-1] = '\0';
 }
 
+/**
+ * prints s then prints the last |t|-|s| characters of t.
+ * if s is longer than t, it truncates s.
+ */
+void left_print_string(FILE * fp, const char * s, const char * t)
+{
+	int i, j, k;
+	j = strlen(t);
+	k = strlen(s);
+	for (i=0; i<j; i++) {
+		if (i<k) {
+			fprintf(fp, "%c", s[i]);
+		} else {
+			fprintf(fp, "%c", t[i]);
+		}
+	}
+}
+
+#ifdef _WIN32  /* should be !defined(HAVE_STRTOK_R) */
+
+char *
+strtok_r (char *s, const char *delim, char **saveptr)
+{
+	char *p;
+
+	if (s == NULL)
+		s = *saveptr;
+
+	if (s == NULL)
+		return NULL;
+
+	/* Skip past any delimiters. */
+	/* while (*s && strchr (delim, *s)) s++; */
+	s += strspn(s, delim);
+
+	if (*s == '\0')
+	{
+		*saveptr = NULL;
+		return NULL;
+	}
+
+	/* Look for end of the token. */
+	/* p = s; while (*p && !strchr (delim, *p)) p++; */
+	p = strpbrk(s, delim);
+	if (p == NULL)
+	{
+		*saveptr = NULL;
+		return s;
+	}
+
+	*p = 0x0;
+	*saveptr = p+1;
+
+	return s;
+}
+
+#endif /* _WIN32 should be !HAVE_STROTOK_R */
+
+/* ============================================================= */
+/* UTF8 utilities */
+
 /**
  * Downcase the first letter of the word.
  */
@@ -72,7 +136,7 @@ void downcase_utf8_str(char *to, const char * from, size_t usize)
 {
 	wchar_t c;
 	int i, nbl, nbh;
-	char low[MB_CUR_MAX];
+	char low[MB_LEN_MAX];
 
 	nbh = mbtowc (&c, from, 4);
 	c = towlower(c);
@@ -103,7 +167,7 @@ void upcase_utf8_str(char *to, const char * from, size_t usize)
 {
 	wchar_t c;
 	int i, nbl, nbh;
-	char low[MB_CUR_MAX];
+	char low[MB_LEN_MAX];
 
 	nbh = mbtowc (&c, from, 4);
 	c = towupper(c);
@@ -127,32 +191,20 @@ void upcase_utf8_str(char *to, const char * from, size_t usize)
 	safe_strcpy(to, from, usize-nbl);
 }
 
-unsigned int randtable[RTSIZE];
-
-/* There is a legitimate question of whether having the hash function	*/
-/* depend on a large array is a good idea.  It might not be fastest on   */
-/* a machine that depends on caching for its efficiency.  On the other   */
-/* hand, Phong Vo's hash (and probably other linear-congruential) is	 */
-/* pretty bad.  So, mine is a "competitive" hash function -- you can't   */
-/* make it perform horribly.											 */
-
-void init_randtable(void) {
-	int i;
-	srand(10);
-	for (i=0; i<RTSIZE; i++) {
-		randtable[i] = rand();
-	}
-}
+/* ============================================================= */
+/* memory alloc routines below */
 
 int max_space_in_use;
 int space_in_use;
 int max_external_space_in_use;
 int external_space_in_use;
 
-void * xalloc(int size) {
-/* To allow printing of a nice error message, and keep track of the
-   space allocated.
-*/
+/**
+ * To allow printing of a nice error message, and keep track of the
+ *  space allocated.
+ */
+void * xalloc(int size)
+{
 	char * p = (char *) malloc(size);
 	space_in_use += size;
 	if (space_in_use > max_space_in_use) max_space_in_use = space_in_use;
@@ -164,13 +216,14 @@ void * xalloc(int size) {
 	return (void *) p;
 }
 
-void xfree(void * p, int size) {
+void xfree(void * p, int size)
+{
 	space_in_use -= size;
 	free(p);
 }
 
-void * exalloc(int size) {
-
+void * exalloc(int size)
+{
 	char * p = (char *) malloc(size);
 	external_space_in_use += size;
 	if (external_space_in_use > max_external_space_in_use) {
@@ -194,27 +247,31 @@ void string_delete(char * p) {
 	exfree(p, strlen(p)+1);
 }
 
-int next_power_of_two_up(int i) {
 /* Returns the smallest power of two that is at least i and at least 1 */
+int next_power_of_two_up(int i)
+{
 	int j=1;
 	while(j<i) j = j<<1;
 	return j;
 }
 
-void left_print_string(FILE * fp, const char * s, const char * t) {
-/* prints s then prints the last |t|-|s| characters of t.
-   if s is longer than t, it truncates s.
-*/
-	int i, j, k;
-	j = strlen(t);
-	k = strlen(s);
-	for (i=0; i<j; i++) {
-		if (i<k) {
-			fprintf(fp, "%c", s[i]);
-		} else {
-			fprintf(fp, "%c", t[i]);
-		}
-	}
+/* =========================================================== */
+/* File path and dictionary open routines below */
+
+char * join_path(const char * prefix, const char * suffix)
+{
+	char * path;
+	int path_len;
+
+	path_len = strlen(prefix) + 1 /* len(DIR_SEPARATOR) */ + strlen(suffix);
+	path = malloc(path_len + 1);
+
+	strcpy(path, prefix);
+	path[strlen(path)+1] = '\0';
+	path[strlen(path)] = DIR_SEPARATOR;
+	strcat(path, suffix);
+
+	return path;
 }
 
 #ifdef _WIN32
@@ -319,6 +376,9 @@ void set_data_dir(const char * path)
 
 static char * get_datadir(void)
 {
+#ifdef _WIN32
+	HINSTANCE hInstance;
+#endif
 	char * data_dir = NULL;
 
 	if (custom_data_dir != NULL) {
@@ -330,7 +390,7 @@ static char * get_datadir(void)
 	data_dir = safe_strdup (BR_DATADIR("/link-grammar"));
 #elif defined(_WIN32)
 	/* Dynamically locate library and return containing directory */
-	HINSTANCE hInstance = GetModuleHandle(NULL);
+	hInstance = GetModuleHandle(NULL);
 	if(hInstance != NULL)
 	{
 		char dll_path[MAX_PATH];
@@ -375,16 +435,22 @@ FILE *dictopen(const char *filename, const char *how)
 	{
 		strncpy (prevpath, filename, MAX_PATH_NAME);
 		prevpath[MAX_PATH_NAME] = 0;
-		pos = strrchr(prevpath, '/');
+		pos = strrchr(prevpath, DIR_SEPARATOR);
 		if (pos) *pos = 0;
-		pos = strrchr(prevpath, '/');
+		pos = strrchr(prevpath, DIR_SEPARATOR);
 		if (pos) *(pos+1) = 0;
 		first_time_ever = 0;
 	}
-	if (filename[0] == '/') {
+
+	/* Look for absolute filename.
+	 * Unix: starts with leading slash.
+	 * Windows: starts with C:\  except that the drive letter may differ.
+	 */
+	if ((filename[0] == '/') || ((filename[1] == ':') && (filename[2] == '\\'))) 
+   {
 		/* fopen returns NULL if the file does not exist. */
 		fp = fopen(filename, how);
-		if(fp) return fp;
+		if (fp) return fp;
 	}
 
 	{
@@ -397,15 +463,20 @@ FILE *dictopen(const char *filename, const char *how)
 		}
 		else {
 			/* Always make sure that it ends with a path
-			 * separator char for the below while() loop. */
+			 * separator char for the below while() loop.
+			 * For unix, this should look like:
+			 * /usr/share/link-grammar:.:data:..:../data:
+			 * For windows:
+			 * C:\SOMWHERE;.;data;..;..\data;
+			 */
 			snprintf(fulldictpath, MAX_PATH_NAME,
-			         "%s%c%s%c%s%c%s%c%s%c%s%c",
+			         "%s%c%s%c%s%c%s%c%s%c%s%c%s%c",
 				 prevpath, PATH_SEPARATOR,
 				 DEFAULTPATH, PATH_SEPARATOR,
 				 ".", PATH_SEPARATOR,
 				 "data", PATH_SEPARATOR,
 				 "..", PATH_SEPARATOR,
-				 "../data", PATH_SEPARATOR);
+				 "..", DIR_SEPARATOR, "data", PATH_SEPARATOR);
 		}
 	}
 
@@ -428,18 +499,44 @@ FILE *dictopen(const char *filename, const char *how)
 	return NULL;
 }
 
+/* ======================================================== */
+/* Random number stuff below. Can this be replaced by 
+ * standard system API's ??
+ */
+
+unsigned int randtable[RTSIZE];
+
+/* There is a legitimate question of whether having the hash function	*/
+/* depend on a large array is a good idea.  It might not be fastest on   */
+/* a machine that depends on caching for its efficiency.  On the other   */
+/* hand, Phong Vo's hash (and probably other linear-congruential) is	 */
+/* pretty bad.  So, mine is a "competitive" hash function -- you can't   */
+/* make it perform horribly.											 */
+
+void init_randtable(void)
+{
+	int i;
+	srand(10);
+	for (i=0; i<RTSIZE; i++) {
+		randtable[i] = rand();
+	}
+}
+
+
 static int random_state[2] = {0,0};
 static int random_count = 0;
 static int random_inited = FALSE;
 
-static int step_generator(int d) {
+static int step_generator(int d)
+{
 	/* no overflow should occur, so this is machine independent */
 	random_state[0] = ((random_state[0] * 3) + d + 104729) % 179424673;
 	random_state[1] = ((random_state[1] * 7) + d + 48611) % 86028121;
 	return random_state[0] + random_state[1];;
 }
 
-void my_random_initialize(int seed) {
+void my_random_initialize(int seed) 
+{
 	assert(!random_inited, "Random number generator not finalized.");
 
 	seed = (seed < 0) ? -seed : seed;
@@ -461,6 +558,9 @@ int my_random(void) {
 	return (step_generator(random_count));
 }
 
+/* ======================================================== */
+/* Locale routines */
+
 #ifdef _WIN32
 
 static char *
@@ -546,22 +646,6 @@ win32_getlocale (void)
 
 #endif
 
-char * join_path(const char * prefix, const char * suffix)
-{
-	char * path;
-	int path_len;
-
-	path_len = strlen(prefix) + 1 /* len(DIR_SEPARATOR) */ + strlen(suffix);
-	path = malloc(path_len + 1);
-
-	strcpy(path, prefix);
-	path[strlen(path)+1] = '\0';
-	path[strlen(path)] = DIR_SEPARATOR;
-	strcat(path, suffix);
-
-	return path;
-}
-
 char * get_default_locale(void)
 {
 	char * locale, * needle;
@@ -603,3 +687,5 @@ char * get_default_locale(void)
 
 	return locale;
 }
+
+/* ========================== END OF FILE =================== */
