@@ -10,9 +10,10 @@
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
 /*************************************************************************/
-#ifndef _UTILITIESH_
-#define _UTILITIESH_
+#ifndef _LINK_GRAMMAR_UTILITIES_H_
+#define _LINK_GRAMMAR_UTILITIES_H_
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <wchar.h>
 #include <wctype.h>
@@ -25,6 +26,41 @@
 #define TRUE 1
 #endif
 
+#ifdef CYGWIN
+#define _WIN32 1
+#endif /* CYGWIN */
+
+#ifdef _WIN32
+#include <windows.h>
+
+#ifdef _MSC_VER
+#define snprintf _snprintf
+#endif
+
+/* Windows compilers don't support the "inline" keyword. */
+#define inline
+
+/* CYGWIN on Windows doesn't have UTF8 support, or wide chars ... 
+ * However, MS Visual C appearnetly does ... 
+ */
+#ifdef CYGWIN
+#define mbtowc(w,s,n)  ({*((char *)(w)) = *(s); 1;})
+#define wctomb(s,w)    ({*((char *)(s)) = ((char)(w)); 1;})
+#define iswupper  isupper
+#define iswalpha  isalpha
+#define iswdigit  isdigit
+#define iswspace  isspace
+#define wchar_t   char
+#define wint_t    int
+#define fgetwc    fgetc
+#define WEOF      EOF
+#endif
+
+/* strtok_r is missing in windows */
+char * strtok_r (char *s, const char *delim, char **saveptr);
+
+#endif /* _WIN32 */
+
 static inline int is_utf8_upper(const char *s)
 {
 	wchar_t c;
@@ -70,7 +106,7 @@ static inline const char * skip_utf8_upper(const char * s)
 
 /**
  * Return true if the intial upper-case letters of the
- * two input strings match. Comparison stops when the 
+ * two input strings match. Comparison stops when
  * both srings descend to lowercase.
  */
 static inline int utf8_upper_match(const char * s, const char * t)
